Imports Microsoft.Win32

' This file handles the events of the application's user interface, including opening a
' data file, displaying the variables it contains, and using the SPSS Processor to
' compute appropriate statistics and charts according to the measurement level of the
' selected variable.  Statistics are displayed in a small grid at the top of the form,
' and a histogram or bar chart is displayed below.
' The user's only choices in this application are the dataset to use and the variable
' to use.  Each time a variable is clicked in the variable list field, results are
' displayed in the main form.

Public Class Explorer
    Public varnames() As System.String
    Public varmls() As System.String
    Public varlabels() As System.String
    Public vartypes() As Integer
    Public Const TEMPDIR = "c:\temp"
    Public Const VARHINTTEXT = "Click on a variable"

    Private Sub OpenFileDialog1_FileOk(ByVal sender As System.Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles SelectDataFile.FileOk
        ' Generate and submit SPSS syntax to open the selected dataset.  Only standard sav files
        ' are supported.

        Dim varCount As Integer
        Me.Label1.Text = sender.filename
        Me.Variables.Items.Clear()

        ' SPSS starts up in an English locale and code page mode when run from .NET.
        ' By setting SPSS to Unicode mode, it will convert the data file correctly to Unicode
        ' if it isn't already a Unicode file and will work in all locales
        ' IF the data file has the encoding recorded in it.
        ' The encoding will be marked with SAV files from SPSS 15 or later.  Otherwise, it will expect
        ' the encoding to be English (cp 1252).

        Module1.p.SPSSP.Submit("SET UNICODE ON.")
        Module1.p.SPSSP.Submit("GET FILE='" + sender.filename + "'.")

        ' Retrieve and save the variable properties that will be used in this application
        ' via programmability apis.  Create a source list of all the variables.

        varCount = Module1.p.SPSSP.GetVariableCount
        ReDim varnames(varCount), varmls(varCount), varlabels(varCount), vartypes(varCount)

        For i As Integer = 0 To varCount - 1
            varnames(i) = Module1.p.SPSSP.GetVariableName(i)
            varmls(i) = Module1.p.SPSSP.GetVariableMeasurementLevel(i)
            varlabels(i) = Module1.p.SPSSP.GetVariableLabel(i)
            If varlabels(i) = "" Then
                varlabels(i) = varnames(i)
            End If
            vartypes(i) = Module1.p.SPSSP.GetVariableType(i)
            Me.Variables.Items.Add(varlabels(i))
        Next
        Me.varhint.Text = VARHINTTEXT
        Me.varhint.Visible = True
        Me.VarlabelValue.Visible = False
        Me.Varlabel.Visible = False
        Me.chart.Visible = False
        Me.ScaleStats.Visible = False
        Me.catstats.Visible = False

    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChooseData.Click
        ' Raise the Open File dialog to select an SPSS data file.

        Me.SelectDataFile.ShowDialog()
    End Sub

    Private Sub Form1_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Leave
        ' Termination processing
        ' When the application closes, it should stop the SPSS Processor.
        Module1.p.SPSSP.StopSPSS()
    End Sub
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Start by creating an instance of the SPSS Processor for use invisibly by this application
        ' Make sure the temporary directory exists.  No error occurs if the directory already exists

        My.Computer.FileSystem.CreateDirectory(TEMPDIR)
        Call Module1.Main()
    End Sub

    Private Sub Variables_SelectedValueChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Variables.SelectedValueChanged
        ' Respond to a click in the variable list and do statistics on the clicked variable.

        Dim var As String, vtype As Integer, lbindex As Integer, vlevel As String
        lbindex = Me.Variables.SelectedIndex()
        var = varnames(lbindex)
        vlevel = varmls(lbindex)
        vtype = vartypes(lbindex)
        Me.varhint.Visible = False
        Me.Cursor = Cursors.WaitCursor
        If vlevel = "Scale" Then
            Call scalevar(var)
        Else
            Call catvar(var, vtype)
        End If

        Me.VarlabelValue.Text = varlabels(lbindex)
        Me.Varlabel.Visible = True
        Me.VarlabelValue.Visible = True
        Me.varhint.Visible = True
        Me.Cursor = Cursors.Default
    End Sub
    Sub scalevar(ByVal var As String)
        ' Do statistics for a variable with a scale measurement level.

        ' var is the name of the variable to use.

        ' Use OMS to capture the output from the EXAMINE command, writing to the XMLWORKSPACE.
        ' Retrieve selected statistics using an XPath expression, and get the chart from a file
        ' For charts, in SPSS 16, it is necessary to know the chart name in advance.
        ' In order to get the tables into the workspace and the chart into a file, it is
        ' necessary to use two simultaneous OMS requests.

        ' In order to understand the XPath expressions below, it is helpful to save the
        ' command output via OMS as an XML file and study the XML with the help of an XML
        ' editor that supports XPath expressions such as XMLSPY.

        Dim oms1 As String = _
          "OMS SELECT TABLES" _
        + "/DESTINATION FORMAT=OXML XMLWORKSPACE='scaleoutput'."
        Dim oms2 As String = _
          "OMS SELECT CHARTS /DESTINATION FORMAT=OXML IMAGES=YES\n" _
        + "IMAGEFORMAT=JPG CHARTSIZE=75 CHARTFORMAT=IMAGE\n" _
        + "OUTFILE = '" + TEMPDIR + "/scale.xml'."
        Dim examine As String = _
          "EXAMINE VARIABLES=" + var + "/PLOT HISTOGRAM\n" _
        + "/STATISTICS DESCRIPTIVES EXTREME(3)."
        Dim oms3 As String = "OMSEND."
        Dim cmds As System.Array = New String() _
        {oms1, oms2, examine, oms3}

        Me.chart.ImageLocation = Nothing
        'Me.chart.Dispose()
        If My.Computer.FileSystem.FileExists(TEMPDIR + "\scale_files\000.jpg") Then
            My.Computer.FileSystem.DeleteFile(TEMPDIR + "\scale_files\000.jpg")
        End If

        Module1.p.SPSSP.Submit(cmds)

        ' take the first two statistics
        Dim XPnAndPct As String = _
        "//pivotTable[@subType='Case Processing Summary']//cell/@text"
        Dim XPMeanAndSEMean As String = _
        "//pivotTable[@subType='Descriptives']//category[@text='Mean']//cell/@text"
        Dim XPMedian As String = _
        "//pivotTable[@subType='Descriptives']//category[@text='Median']//cell/@text"
        Dim XPStddev As String = _
        "//pivotTable[@subType='Descriptives']//category[@text='Std. Deviation']//cell/@text"
        Dim XPChartImage As String = TEMPDIR + "/scale/000.jpg"

        Dim nAndPct() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPnAndPct)
        Dim MeanAndSEMean() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPMeanAndSEMean)
        Dim Stddev() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPStddev)
        Dim Median() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPMedian)
        If Me.ScaleStats.Rows.Count = 0 Then
            Me.ScaleStats.Rows.Add()
        End If
        Me.ScaleStats.Item(0, 0).Value = nAndPct(0)
        Me.ScaleStats.Item(1, 0).Value = nAndPct(1)
        Me.ScaleStats.Item(2, 0).Value = MeanAndSEMean(0)
        Me.ScaleStats.Item(3, 0).Value = Median(0)
        Me.ScaleStats.Item(4, 0).Value = Stddev(0)
        Me.catstats.Visible = False
        Me.ScaleStats.Visible = True

        ' The generated chart is picked up from a file.  The OMS request
        ' scales the chart to 75% of its normal size.  However, the actual
        ' chart size can vary compared to the area allocated to the chart control
        ' on the form.  To handle this, the control property SizeMode is set to
        ' StretchImage on the form.

        Me.chart.ImageLocation = TEMPDIR + "\scale_files\000.jpg"
        Me.Greeting.Visible = False
        Me.chart.Visible = True


    End Sub
    Sub catvar(ByVal var As String, ByVal vtype As Integer)
        ' Do statistics on a categorical variable.

        ' var is the name of the variable to use.
        ' vtype is the integer type code for the variable.  This is needed
        ' in order to extract the correct information from the Frequency table.

        ' Run FREQUENCIES and retrieve selected statistics and a chart.
        ' See the comments in sub scalevar for additional information.

        Dim oms1 As String = _
                  "OMS SELECT TABLES" _
                + "/DESTINATION FORMAT=OXML XMLWORKSPACE='scaleoutput'."
        Dim oms2 As String = _
          "OMS SELECT CHARTS /DESTINATION FORMAT=OXML IMAGES=YES\n" _
        + "IMAGEFORMAT=JPG CHARTSIZE=75 CHARTFORMAT=IMAGE\n" _
        + "OUTFILE = '" + TEMPDIR + "/scale.xml'."
        Dim freq As String = _
          "FREQUENCIES VARIABLES=" + var _
          + "/FORMAT DFREQ/BARCHART FREQ."
        Dim oms3 As String = "OMSEND."
        Dim cmds As System.Array = New String() _
        {oms1, oms2, freq, oms3}

        Me.chart.ImageLocation = Nothing
        'Me.chart.Dispose()
        If My.Computer.FileSystem.FileExists(TEMPDIR + "\scale_files\000.jpg") Then
            My.Computer.FileSystem.DeleteFile(TEMPDIR + "\scale_files\000.jpg")
        End If

        Module1.p.SPSSP.Submit(cmds)
        Dim XPcategories As String
        If vtype > 0 Then
            XPcategories = "//pivotTable[@subType='Frequencies']//group[@text='Valid']/group/group/category/@text"
        Else
            XPcategories = "//pivotTable[@subType='Frequencies']//group[@text='Valid']/group/group/category/@text"
        End If  'XPcategories = "//pivotTable[@subType='Frequencies']//group[@text='Valid']//category/@label"

        Dim categories() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPcategories)

        Dim XPcounts As String
        If vtype > 0 Then
            XPcounts = _
            "//pivotTable[@subType='Frequencies']//group[@text='Valid']/group//category[@text='Frequency']/cell/@text"
        Else
            XPcounts = "//pivotTable[@subType='Frequencies']//group[@text='Valid']/group/group/category//category[@text='Frequency']/cell/@text"
        End If
        Dim counts() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPcounts)

        Dim XPvalidMissing As String = _
            "//pivotTable[@subType='Statistics']//cell/@text"
        Dim validMissing() As String = _
            Module1.p.SPSSP.EvaluateXPath("scaleoutput", "/", XPvalidMissing)

        If Me.catstats.Rows.Count = 0 Then
            Me.catstats.Rows.Add(2)
        End If
        Me.catstats.Item(0, 1).Value = validMissing(0)
        Dim valid As Double = validMissing(0), missing As Double = validMissing(1)
        Me.catstats.Item(1, 1).Value = Format(valid / (valid + missing), "Percent")

        Dim i As Integer, numcat As Integer = UBound(counts)

        For i = 0 To 2
            If i <= numcat Then
                Me.catstats.Item(2 + i, 0).Value = categories(i)
                Me.catstats.Item(2 + i, 1).Value = counts(i)
            Else
                Me.catstats.Item(2 + i, 0).Value = ""
                Me.catstats.Item(2 + i, 1).Value = ""
            End If
        Next
        Me.ScaleStats.Visible = False
        Me.catstats.Visible = True
        Me.chart.ImageLocation = TEMPDIR + "\scale_files\000.jpg"
        Me.Greeting.Visible = False
        Me.chart.Visible = True
        Me.Variables.Focus()

    End Sub

    Private Sub Label3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Label3.Click

    End Sub
End Class
